import * as React from "react";
import { IModule, IModuleStore } from "./Contracts";
export interface IDynamicModuleLoaderProps<OriginalState, AdditionalState> {
    /** Modules that need to be dynamically registerd */
    modules: IModule<AdditionalState>[];
}
export interface IDynamicModuleLoaderContext {
    store: IModuleStore<any>;
}
/**
 * The DynamicModuleLoader adds a way to register a module on mount
 * When this component is initialized, the reducer and saga from the module passed as props will be registered with the system
 * On unmount, they will be unregistered
 */
export declare class DynamicModuleLoader<OriginalState, AdditionalState> extends React.Component<IDynamicModuleLoaderProps<OriginalState, AdditionalState>> {
    private _addedModules?;
    private static contextTypes;
    constructor(props: IDynamicModuleLoaderProps<OriginalState, AdditionalState>, context: IDynamicModuleLoaderContext);
    /**
     * Unregister sagas and reducers
     */
    componentWillUnmount(): void;
    /**
     * Render a Redux provider
     */
    render(): React.ReactNode;
}
